<?php
declare(strict_types=1);

use Migrations\AbstractMigration;

/**
 * Class ScrapeTimeout
 *
 * Created:
 * oitc migrations create -p PrometheusModule ScrapeTimeout
 *
 * Usage:
 * openitcockpit-update
 */
class ScrapeTimeout extends AbstractMigration {

    /**
     * Whether the tables created in this migration
     * should auto-create an `id` field or not
     *
     * This option is global for all tables created in the migration file.
     * If you set it to false, you have to manually add the primary keys for your
     * tables using the Migrations\Table::addPrimaryKey() method
     *
     * @var bool
     */
    public $autoId = false;

    /**
     * Change Method.
     *
     * More information on this method is available here:
     * https://book.cakephp.org/phinx/0/en/migrations.html#the-change-method
     * @return void
     */
    public function change(): void {

        $this->table('prometheus_exporters')
            ->addColumn('scrape_timeout', 'string', [
                'after'   => 'scrape_interval',
                'default' => '15s',
                'limit'   => 128,
                'null'    => false
            ])
            ->update();

        $table = $this->table('hosttemplates_to_prometheus_exporters');
        $table
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default'       => null,
                'limit'         => 10,
                'null'          => false,
            ])
            ->addPrimaryKey(['id']);
        $table
            ->addColumn('prometheus_exporter_id', 'integer', [
                'null' => false,
            ])
            ->addColumn('hosttemplate_id', 'integer', [
                'null' => false,
            ])
            ->create();

        $table = $this->table('hosts_to_prometheus_exporters');
        $table
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default'       => null,
                'limit'         => 10,
                'null'          => false,
            ])
            ->addPrimaryKey(['id']);
        $table
            ->addColumn('prometheus_exporter_id', 'integer', [
                'null' => false,
            ])
            ->addColumn('host_id', 'integer', [
                'null' => false,
            ])
            ->create();
    }

}
