<?php
declare(strict_types=1);

use Migrations\AbstractMigration;

/**
 * Class AlertRules
 *
 * Created:
 * oitc migrations create -p PrometheusModule AlertRules
 *
 * Usage:
 * openitcockpit-update
 */
class AlertRules extends AbstractMigration {

    /**
     * Whether the tables created in this migration
     * should auto-create an `id` field or not
     *
     * This option is global for all tables created in the migration file.
     * If you set it to false, you have to manually add the primary keys for your
     * tables using the Migrations\Table::addPrimaryKey() method
     *
     * @var bool
     */
    public $autoId = false;

    /**
     * Change Method.
     *
     * More information on this method is available here:
     * https://book.cakephp.org/phinx/0/en/migrations.html#the-change-method
     * @return void
     */
    public function change(): void {
        $table = $this->table('prometheus_alert_rules');
        $table
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default'       => null,
                'limit'         => 10,
                'null'          => false,
            ])
            ->addPrimaryKey(['id']);
        $table
            ->addColumn('host_id', 'integer', [
                'null' => false,
            ])
            ->addColumn('service_id', 'integer', [
                'null' => false,
            ])
            ->addColumn('promql', 'string', [
                'default' => '',
                'limit'   => 2500,
                'null'    => false,
            ])
            ->addColumn('unit', 'string', [
                'default' => '',
                'limit'   => 255,
                'null'    => false,
            ])
            ->addColumn('threshold_type', 'string', [
                'default' => 'scalar',
                'limit'   => 128,
                'null'    => false,
            ])
            ->addColumn('warning_min', 'float', [
                'null' => false
            ])
            ->addColumn('warning_max', 'float', [
                'null' => true
            ])
            ->addColumn('critical_min', 'float', [
                'null' => false
            ])
            ->addColumn('critical_max', 'float', [
                'null' => true
            ])
            ->addColumn('warning_longer_as', 'string', [
                'default' => '60s',
                'limit'   => 128,
                'null'    => false,
            ])
            ->addColumn('critical_longer_as', 'string', [
                'default' => '60s',
                'limit'   => 128,
                'null'    => false,
            ])
            ->addColumn('warning_operator', 'string', [
                'default' => null,
                'limit'   => 10,
                'null'    => true,
            ])
            ->addColumn('critical_operator', 'string', [
                'default' => null,
                'limit'   => 10,
                'null'    => true,
            ])
            ->addColumn('created', 'datetime', [
                'default' => null,
                'limit'   => null,
                'null'    => false,
            ])
            ->addColumn('modified', 'datetime', [
                'default' => null,
                'limit'   => null,
                'null'    => false,
            ])
            ->create();

    }
}
