<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

use Migrations\AbstractSeed;

/**
 * Class InstallSeed
 *
 * Created:
 * oitc bake seed -p PrometheusModule --table prometheus_exporters --data Install
 *
 * Apply:
 * oitc migrations seed -p PrometheusModule
 */
class InstallSeed extends AbstractSeed {

    /**
     * Run Method.
     *
     * Write your database seeder using this method.
     *
     * More information on writing seeds is available here:
     * http://docs.phinx.org/en/latest/seeding.html
     *
     * @return void
     */
    public function run(): void {
        //Commands
        $table = $this->table('prometheus_exporters');

        $data = [
            [
                'name'            => 'Node exporter',
                'container_id'    => ROOT_CONTAINER,
                'metric_path'     => '/metrics',
                'port'            => 9100,
                'scrape_interval' => '15s',
                'scrape_timeout'  => '15s',
                'service'         => 'os',
                'yaml'            => '',
                'add_target_port' => 1
            ],
            [
                'name'            => 'Windows exporter',
                'container_id'    => ROOT_CONTAINER,
                'metric_path'     => '/metrics',
                'port'            => 9182,
                'scrape_interval' => '15s',
                'scrape_timeout'  => '15s',
                'service'         => 'windows',
                'yaml'            => '',
                'add_target_port' => 1
            ],
            [
                'name'            => 'Blackbox exporter',
                'container_id'    => ROOT_CONTAINER,
                'metric_path'     => '/probe',
                'port'            => 9115,
                'scrape_interval' => '30s',
                'scrape_timeout'  => '15s',
                'service'         => 'blackbox',
                'yaml'            => "params:
  module: [icmp]
relabel_configs:
  - source_labels: [__address__]
    target_label: __param_target
  - target_label: __address__
    replacement: '127.0.0.1:9115'",
                'add_target_port' => 0
            ],
            [
                'name'            => 'SNMP exporter',
                'container_id'    => ROOT_CONTAINER,
                'metric_path'     => '/snmp',
                'port'            => 9115,
                'scrape_interval' => '30s',
                'scrape_timeout'  => '15s',
                'service'         => 'snmp',
                'yaml'            => "params:
  module: [if_mib]
relabel_configs:
  - source_labels: [__address__]
    target_label: __param_target
  - source_labels: [__param_target]
    target_label: instance
  - target_label: __address__
    replacement: 127.0.0.1:9116  # The SNMP exporter's real hostname:port.",
                'add_target_port' => 0
            ]
        ];

        //Check if records exists
        foreach ($data as $index => $record) {
            $QueryBuilder = $this->getAdapter()->getQueryBuilder();

            $stm = $QueryBuilder->select('*')
                ->from($table->getName())
                ->where([
                    'name' => $record['name']
                ])
                ->execute();
            $result = $stm->fetchAll();

            if (empty($result)) {
                $table->insert($record)->save();
            }
        }

        //Check for Prometheus commands and service templates
        $PrometheusImporter = new \PrometheusModule\Lib\InitialDatabase\Prometheus(
            \Cake\ORM\TableRegistry::getTableLocator()->get('Commands'),
            \Cake\ORM\TableRegistry::getTableLocator()->get('Servicetemplates')
        );
        $PrometheusImporter->import();
    }
}
