<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace PrometheusModule\Controller;


use App\Model\Table\ConfigurationFilesTable;
use App\Model\Table\ConfigurationQueueTable;
use Cake\Http\Exception\MethodNotAllowedException;
use Cake\Http\Exception\NotFoundException;
use Cake\ORM\TableRegistry;
use Exception;
use itnovum\openITCOCKPIT\ConfigGenerator\ConfigInterface;

class ConfigurationFilesController extends AppController {
    /**
     * @throws Exception
     */
    public function PrometheusCfgsPrometheus() {
        $this->__sharedControllerAction('\PrometheusModule\Lib\ConfigGenerator\PrometheusCfgs_prometheus', 'PrometheusCfgs_prometheus');
    }

    /**
     * @param $configFile
     * @throws \Exception
     */
    public function restorDefault($configFile) {
        if (!$this->request->is('post') || !$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        $className = sprintf('\PrometheusModule\Lib\ConfigGenerator\%s', $configFile);
        if (!class_exists($className)) {
            throw new NotFoundException('Config file not found');
        }

        /** @var ConfigurationFilesTable $ConfigurationFilesTable */
        $ConfigurationFilesTable = TableRegistry::getTableLocator()->get('ConfigurationFiles');
        /** @var ConfigurationQueueTable $ConfigurationQueueTable */
        $ConfigurationQueueTable = TableRegistry::getTableLocator()->get('ConfigurationQueue');

        /** @var ConfigInterface $ConfigurationObjectClassName */
        $ConfigurationObjectClassName = new $className();

        $currentConfig = $ConfigurationFilesTable->getConfigValuesByConfigFile($ConfigurationObjectClassName->getDbKey());
        $config = $ConfigurationObjectClassName->convertRequestForSaveAll($ConfigurationObjectClassName->getDefaults());

        if ($ConfigurationFilesTable->saveConfigurationValuesForConfigFile($ConfigurationObjectClassName->getDbKey(), $config)) {
            $configHasChanged = $ConfigurationFilesTable->hasChanged($currentConfig, $config);

            if ($configHasChanged) {
                //Require rewirte of configuration file on disk?
                $ConfigurationQueueTable->deleteAll([
                    'task' => 'ConfigGenerator',
                    'data' => $ConfigurationObjectClassName->getDbKey()
                ]);

                $queueEntity = $ConfigurationQueueTable->newEntity([
                    'task'   => 'ConfigGenerator',
                    'data'   => $ConfigurationObjectClassName->getDbKey(),
                    'module' => 'PrometheusModule'
                ]);

                $ConfigurationQueueTable->save($queueEntity);
            }

            $this->set('success', true);
            $this->set('message', __('Config successfully restored to default'));
            $this->viewBuilder()->setOption('serialize', ['success', 'message']);
            return;
        }

        $this->response = $this->response->withStatus(400);
        $this->set('success', false);
        $this->viewBuilder()->setOption('serialize', ['success']);
        return;
    }

    /**
     * @param $ConfigurationObjectClassName
     * @param $ShortClassName
     * @throws Exception
     */
    private function __sharedControllerAction($ConfigurationObjectClassName, $ShortClassName) {
        $ConfigurationObjectClassName = new $ConfigurationObjectClassName();
        $this->set($ShortClassName, $ConfigurationObjectClassName);

        /** @var ConfigurationFilesTable $ConfigurationFilesTable */
        $ConfigurationFilesTable = TableRegistry::getTableLocator()->get('ConfigurationFiles');

        if ($this->request->is('get') && $this->isAngularJsRequest()) {
            $dbConfig = $ConfigurationFilesTable->getConfigValuesByConfigFile($ConfigurationObjectClassName->getDbKey());
            $config = $ConfigurationObjectClassName->mergeDbResultWithDefaultConfiguration($dbConfig);

            $this->set('config', $config);
            $this->viewBuilder()->setOption('serialize', ['config']);
        }

        if ($this->request->is('post')) {
            /** @var ConfigurationQueueTable $ConfigurationQueueTable */
            $ConfigurationQueueTable = TableRegistry::getTableLocator()->get('ConfigurationQueue');

            if ($ConfigurationObjectClassName->validate($this->request->getData())) {
                //Save new config to database
                $configFileForDatabase = $ConfigurationObjectClassName->convertRequestForSaveAll($this->request->getData());

                $currentConfig = $ConfigurationFilesTable->getConfigValuesByConfigFile($ConfigurationObjectClassName->getDbKey());

                $configHasChanged = $ConfigurationFilesTable->hasChanged($currentConfig, $configFileForDatabase);

                if ($ConfigurationFilesTable->saveConfigurationValuesForConfigFile($ConfigurationObjectClassName->getDbKey(), $configFileForDatabase)) {
                    $this->set('success', true);
                    $this->viewBuilder()->setOption('serialize', ['success']);

                    if ($configHasChanged) {
                        //Require rewirte of configuration file on disk?
                        $ConfigurationQueueTable->deleteAll([
                            'task' => 'ConfigGenerator',
                            'data' => $ConfigurationObjectClassName->getDbKey()
                        ]);

                        $queueEntity = $ConfigurationQueueTable->newEntity([
                            'task' => 'ConfigGenerator',
                            'data' => $ConfigurationObjectClassName->getDbKey(),
                            'module' => 'PrometheusModule'
                        ]);

                        $ConfigurationQueueTable->save($queueEntity);
                    }

                    return;
                } else {
                    $this->response = $this->response->withStatus(400);
                    $this->set('success', false);
                    $this->viewBuilder()->setOption('serialize', ['success']);
                    return;
                }

            } else {
                $this->response = $this->response->withStatus(400);
                $error = $ConfigurationObjectClassName->validationErrors;
                $this->set('error', $error);
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            }
        }
    }
}