<?php


namespace PrometheusModule\Lib;


use Cake\Validation\Validator;
use itnovum\openITCOCKPIT\Core\FileDebugger;

class AlertRuleValidationRules {

    /**
     * @param Validator $validator
     * @return Validator
     */
    public static function attachRules(Validator $validator): Validator {
        $ForLengths = new ForLengths();
        $thresholdTypes = [
            'scalar',
            'range_inclusive',
            'range_exclusive'
        ];
        $operators = [
            '==',
            '!=',
            '>',
            '>=',
            '<',
            '<='
        ];


        $validator
            ->notEmptyString('promql')
            ->requirePresence('promql', 'create');

        $validator
            ->allowEmptyString('unit');

        $validator
            ->scalar('threshold_type')
            ->maxLength('threshold_type', 128)
            ->requirePresence('threshold_type', 'create')
            ->inList('threshold_type', $thresholdTypes, __('threshold type as is out of bounds: [{0}]', implode(', ', $thresholdTypes)))
            ->notEmptyString('threshold_type');

        $validator
            ->requirePresence('warning_min', 'create')
            ->numeric('warning_min')
            ->notEmptyString('warning_min')
            ->add('warning_min', 'not_equal', [
                'rule'    => function ($value, $context) {
                    if (isset($context['data']['threshold_type']) && $context['data']['threshold_type'] === 'scalar') {
                        if (!empty($context['data']['warning_operator']) && !empty($context['data']['critical_operator'])) {
                            $isOneOperatorEqual = false;
                            if (in_array($context['data']['warning_operator'], ['==', '!=', '>=', '<='], true)) {
                                $isOneOperatorEqual = true;
                            }

                            if (in_array($context['data']['critical_operator'], ['==', '!=', '>=', '<='], true)) {
                                $isOneOperatorEqual = true;
                            }

                            if ($isOneOperatorEqual === true) {
                                //Warning and critical can not be the same value !
                                return $context['data']['warning_min'] != $context['data']['critical_min'];
                            }
                        }
                    }

                    //Not scalar
                    return true;
                },
                'message' => __('Warning and Critical can not be equal.')
            ]);

        $validator
            ->requirePresence('warning_max', 'create')
            ->numeric('warning_max')
            ->allowEmptyString('warning_max', null, function ($context) {
                if (isset($context['data']['threshold_type']) && $context['data']['threshold_type'] === 'scalar') {
                    //Only required for range thresholds
                    return true;
                }
                return false;
            });

        $validator
            ->requirePresence('critical_min', 'create')
            ->numeric('critical_min')
            ->notEmptyString('critical_min')
            ->add('critical_min', 'not_equal', [
                'rule'    => function ($value, $context) {
                    if (isset($context['data']['threshold_type']) && $context['data']['threshold_type'] === 'scalar') {
                        if (!empty($context['data']['warning_operator']) && !empty($context['data']['critical_operator'])) {
                            $isOneOperatorEqual = false;
                            if (in_array($context['data']['warning_operator'], ['==', '!=', '>=', '<='], true)) {
                                $isOneOperatorEqual = true;
                            }

                            if (in_array($context['data']['critical_operator'], ['==', '!=', '>=', '<='], true)) {
                                $isOneOperatorEqual = true;
                            }

                            if ($isOneOperatorEqual === true) {
                                //Warning and critical can not be the same value !
                                return $context['data']['warning_min'] != $context['data']['critical_min'];
                            }
                        }
                    }

                    //Not scalar
                    return true;
                },
                'message' => __('Warning and Critical can not be equal.')
            ]);

        $validator
            ->requirePresence('critical_max', 'create')
            ->numeric('critical_max')
            ->allowEmptyString('critical_max', null, function ($context) {
                if (isset($context['data']['threshold_type']) && $context['data']['threshold_type'] === 'scalar') {
                    //Only required for range thresholds
                    return true;
                }
                return false;
            });

        $validator
            ->scalar('warning_longer_as')
            ->maxLength('warning_longer_as', 128)
            ->requirePresence('warning_longer_as', 'create')
            ->inList('warning_longer_as', array_keys($ForLengths->getLengths()), __('Longer as is out of bounds: [{0}]', implode(', ', array_keys($ForLengths->getLengths()))))
            ->notEmptyString('warning_longer_as');

        $validator
            ->scalar('critical_longer_as')
            ->maxLength('critical_longer_as', 128)
            ->requirePresence('critical_longer_as', 'create')
            ->inList('critical_longer_as', array_keys($ForLengths->getLengths()), __('Longer as is out of bounds: [{0}]', implode(', ', array_keys($ForLengths->getLengths()))))
            ->notEmptyString('critical_longer_as');

        $validator
            ->inList('warning_operator', $operators, __('Operator as is out of bounds: [{0}]', implode(', ', $operators)))
            ->allowEmptyString('warning_operator', null, function ($context) {
                if (isset($context['data']['threshold_type']) && $context['data']['threshold_type'] !== 'scalar') {
                    //Only required for scalar thresholds
                    return true;
                }
                return false;
            });

        $validator
            ->inList('critical_operator', $operators, __('Operator as is out of bounds: [{0}]', implode(', ', $operators)))
            ->allowEmptyString('critical_operator', null, function ($context) {
                if (isset($context['data']['threshold_type']) && $context['data']['threshold_type'] !== 'scalar') {
                    //Only required for scalar thresholds
                    return true;
                }
                return false;
            });

        return $validator;
    }

}