<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.


namespace PrometheusModule\Lib\ConfigGenerator;


use itnovum\openITCOCKPIT\ConfigGenerator\ConfigGenerator;
use itnovum\openITCOCKPIT\ConfigGenerator\ConfigInterface;
use itnovum\openITCOCKPIT\ConfigGenerator\ConfigSymlink;
use itnovum\openITCOCKPIT\ConfigGenerator\FileHeader;
use PrometheusModule\Lib\PrometheusConfigGenerator;
use Twig\Environment;
use Twig\Error\LoaderError;
use Twig\Error\RuntimeError;
use Twig\Error\SyntaxError;
use Twig\Loader\FilesystemLoader;

class PrometheusCfgs_prometheus extends ConfigGenerator implements ConfigInterface {

    protected $basePath = PLUGIN . 'PrometheusModule' . DS . 'src' . DS;

    protected $templateDir = 'config_templates';

    protected $template = 'prometheus_bridge.php.tpl';

    protected $realOutfile = PLUGIN . 'PrometheusModule' . DS . 'config' . DS . 'prometheus_bridge.php';

    protected $linkedOutfile = PLUGIN . 'PrometheusModule' . DS . 'config' . DS . 'prometheus_bridge.php';

    /**
     * @var string
     */
    protected $commentChar = '//';

    protected $defaults = [
        'int' => [
            'loop_interval' => 15
        ]
    ];

    protected $dbKey = 'PrometheusCfgs_prometheus';

    /**
     * @param array $data
     * @return array|bool|true
     */
    public function customValidationRules($data) {
        return true;
    }

    /**
     * @return string
     */
    public function getAngularDirective() {
        return 'prometheus-cfgs-prometheus';
    }

    /**
     * @param string $key
     * @return string
     */
    public function getHelpText($key) {
        $help = [
            'loop_interval' => 'Loop interval of the prometheus_bridge daemon in seconds'
        ];

        if (isset($help[$key])) {
            return $help[$key];
        }

        return '';
    }

    /**
     * Save the configuration as text file on disk
     *
     * @param array $dbRecords
     * @return bool|int
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function writeToFile($dbRecords) {
        $config = $this->mergeDbResultWithDefaultConfiguration($dbRecords);
        $configToExport = [];
        foreach ($config as $type => $fields) {
            foreach ($fields as $key => $value) {
                $configToExport[$key] = $value;
            }
        }

        $success = true;

        $FileHeader = new FileHeader();
        $configToExport['STATIC_FILE_HEADER'] = $FileHeader->getHeader($this->commentChar);


        $loader = new FilesystemLoader([
            $this->getTemplatePath()
        ]);
        $twig = new Environment($loader, ['debug' => true]);

        // /opt/openitc/frontend/plugins/PrometheusModule/config/prometheus_bridge.php
        $ConfigSymlink = new ConfigSymlink($this->realOutfile, $this->linkedOutfile);
        if (!file_put_contents($this->realOutfile, $twig->render($this->getTemplateName(), $configToExport))) {
            $success = false;
        }
        $ConfigSymlink->link();

        // /opt/openitc/prometheus/prometheus.yml
        $PrometheusConfigGenerator = new PrometheusConfigGenerator();
        $PrometheusConfigGenerator->createPrometheusYml();

        return $success;

    }

    /**
     * @param array $dbRecords
     * @return bool|array
     */
    public function migrate($dbRecords) {
        return $this->mergeDbResultWithDefaultConfiguration($dbRecords);
    }

}
