<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace PrometheusModule\Lib;

class PromQl {

    /**
     * This method generates the PromQL from a Prometheus query result
     * Like: node_network_up{device="br-c7d3ed45b19c", instance="prometheus", job="15s_15s", service="fake", uuid="4d94bdcc-ed6d-41ae-8749-146e2093799a"}
     *
     * @param array $metric
     * @return string
     */
    public static function getPromQlFromMetric(array $metric): string {
        // Some PromLQ like rate(node_network_up[1m]) have no __name__ key
        $promql = $metric['__name__'] ?? '';

        $labels = [];
        foreach ($metric as $labelKey => $labelValue) {
            if ($labelKey !== '__name__') {
                $labels[] = sprintf('%s="%s"', $labelKey, $labelValue);
            }
        }

        if (empty($labels)) {
            // Only "node_network_up" was hte PromQL
            return $promql;
        }

        // node_network_up{device="br-c7d3ed45b19c", instance="prometheus", job="15s_15s", service="fake", uuid="4d94bdcc-ed6d-41ae-8749-146e2093799a"}
        return sprintf('%s{%s}', $promql, implode(',', $labels));
    }

}
