<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace PrometheusModule\Lib;


use PrometheusModule\Model\Entity\PrometheusAlertRule;

/**
 * Class PrometheusAlertRuleGenerator
 * @package PrometheusModule\Lib
 *
 * This code is way to complex and therefor not used.
 * It is here for reference reasons and may be delete in the future.
 */
class PrometheusAlertRuleGenerator {

    /**
     * @var PrometheusAlertRule
     */
    private $AlertRule;

    /**
     * PrometheusAlertRuleGenerator constructor.
     * @param PrometheusAlertRule $AlertRule
     */
    public function __construct(PrometheusAlertRule $AlertRule) {
        $this->AlertRule = $AlertRule;
    }

    public function getAlertRuleExpressionForConfig() {
        if ($this->AlertRule->threshold_type === 'scalar') {
            return $this->getScalarAlertRuleExpressionForConfig();
        }
    }

    /**
     * @return array
     */
    private function getScalarAlertRuleExpressionForConfig() {
        /**
         * List of operators
         *  ==
         *  !=
         *  >
         *  <
         *  >=
         *  <=
         *
         * Possible operator combinations (36)
         *  == & ==  |  != & ==  |  > & ==  |  < & ==  |  >= & ==  |  <= & ==
         *  == & !=  |  != & !=  |  > & !=  |  < & !=  |  >= & !=  |  <= & !=
         *  == & >   |  != & >   |  > & >   |  < & >   |  >= & >   |  <= & >
         *  == & <   |  != & <   |  > & <   |  < & <   |  >= & <   |  <= & <
         *  == & >=  |  != & >=  |  > & >=  |  < & >=  |  >= & >=  |  <= & >=
         *  == & <=  |  != & <=  |  > & <?  |  < & <=  |  >= & <=  |  <= & <=
         */
        if ($this->AlertRule->warning_operator === '==' || $this->AlertRule->critical_operator === '==') {
            return [
                'warning_rule'  => sprintf(
                    '%s %s %s',
                    $this->AlertRule->promql,
                    $this->AlertRule->warning_operator,
                    $this->AlertRule->warning_min
                ),
                'critical_rule' => sprintf(
                    '%s %s %s',
                    $this->AlertRule->promql,
                    $this->AlertRule->critical_operator,
                    $this->AlertRule->critical_min
                )
            ];
        }

        if ($this->AlertRule->critical_operator === '!=') {
            // IF both are not equal - than we can only have an critical threshold
            return [
                'warning_rule'  => null,
                'critical_rule' => sprintf(
                    '%s %s %s',
                    $this->AlertRule->promql,
                    $this->AlertRule->critical_operator,
                    $this->AlertRule->critical_min
                )
            ];
        }

        if ($this->AlertRule->warning_operator === '!=') {
            // Critical is worst than warning
            // Warning Rule: != 20 //Warning if value is not 20 and less than 30
            // Critical Rule > 30 //Critical if > 30


            // Warn != 20
            // Crit < 30

            return [
                'warning_rule'  => sprintf(
                    '%s != %s and %s %s',
                    $this->AlertRule->promql,
                    $this->AlertRule->warning_min,
                    $this->negateOperator($this->AlertRule->critical_operator),
                    $this->AlertRule->critical_min
                ),
                'critical_rule' => sprintf(
                    '%s %s %s and != %s',
                    $this->AlertRule->promql,
                    $this->AlertRule->critical_operator,
                    $this->AlertRule->critical_min,
                    $this->AlertRule->warning_min
                )
            ];
        }


        if ($this->AlertRule->warning_min <= $this->AlertRule->critical_min) {
            $warningOperator = $this->AlertRule->warning_operator;
            $criticalOperator = $this->AlertRule->critical_operator;

            if ($warningOperator === '>' && $criticalOperator === '!=') {
                //Warning: promql > $warning_min and != $critical_min
                //Critical: promql < $warning_min or != $critical_min
                return [
                    'warning_rule'  => sprintf(
                        '%s > %s and != %s',
                        $this->AlertRule->promql,
                        $this->AlertRule->warning_min,
                        $this->AlertRule->critical_min
                    ),
                    'critical_rule' => sprintf(
                        '%s < %s or != %s',
                        $this->AlertRule->promql,
                        $this->AlertRule->warning_operator,
                        $this->AlertRule->critical_min
                    )
                ];
            }

            if ($warningOperator === '!=' && $criticalOperator === '>') {
                //Warning: promql != $warning_min and < $critical_min
                //Critical: promql < $warning_min or != $critical_min
                return [
                    'warning_rule'  => sprintf(
                        '%s != %s and <= %s',
                        $this->AlertRule->promql,
                        $this->AlertRule->warning_min,
                        $this->AlertRule->critical_min
                    ),
                    'critical_rule' => sprintf(
                        '%s > %s',
                        $this->AlertRule->promql,
                        $this->AlertRule->critical_min
                    )
                ];
            }


        } else {
            //Hard drives maybe???
        }

    }


    /**
     * @param string $operator
     * @return string
     */
    private function negateOperator(string $operator) {
        if ($operator === '==' || $operator === '!=') {
            throw new \RuntimeException('Operator not suported!');
        }

        if ($operator === '>') {
            return '<=';
        }

        if ($operator === '>=') {
            return '<';
        }

        if ($operator === '<') {
            return '>=';
        }

        if ($operator === '<=') {
            return '>';
        }

        throw new \RuntimeException('Operator not suported!');
    }

}
