#!/bin/bash

# Class named 'PrometheusModule' for UPDATE.sh bash object emulation

# property
properties=()

# properties IDs (only placeholder with incrementing array position value)
name=0
disableFileLock=1

# predefined properties
properties[name]='PrometheusModule'
# set 1 to disable file lock, this disables the uninstall() method and will run install() on every openitcockpit-update if the module folder exists
properties[disableFileLock]=1



# do not change this method!
PrometheusModule.property(){
    if [ "$2" == "=" ]; then
        properties[$1]=$3
    else
        echo ${properties[$1]}
    fi
}

# runs on every openitcockpit-update
PrometheusModule.initialize(){
    PrometheusModule.installIfNeeded
}

PrometheusModule.installIfNeeded(){
    if [ $(PrometheusModule.property disableFileLock) == 1 ]; then
        if [ $(system.moduleSrcDirExists $(PrometheusModule.property name)) == 1 ]; then
            PrometheusModule.install
        fi
    else
        if [ $(system.moduleSrcDirExists $(PrometheusModule.property name)) == 1 ]; then
            if [ "$(system.isLocked $(PrometheusModule.property name) 'installed')" != 1 ]; then
                PrometheusModule.install
            fi
        else
            if [ "$(system.isLocked $(PrometheusModule.property name) 'installed')" == 1 ]; then
                PrometheusModule.uninstall
            fi
        fi
    fi
}

PrometheusModule.install(){
    echo "Run $(PrometheusModule.property name) update"
    cp -r /opt/openitc/frontend/plugins/$(PrometheusModule.property name)/system/lib/. /lib/
    systemctl daemon-reload

    if [ -f "/opt/openitc/prometheus/prometheus" ] ; then
        chmod +x /opt/openitc/prometheus/prometheus
    fi

    if [ -f "/opt/openitc/prometheus/promtool" ] ; then
        chmod +x /opt/openitc/prometheus/promtool
    fi

    if [ -f "/opt/openitc/prometheus/tsdb" ] ; then
        chmod +x /opt/openitc/prometheus/tsdb
    fi

    if [ ! -d directory ]; then
        mkdir -p /opt/openitc/var/prometheus/data
    fi

    chown -R nagios:nagios /opt/openitc/var/prometheus/data

    system.lock $(PrometheusModule.property name) "installed" $(PrometheusModule.property disableFileLock)
}

PrometheusModule.uninstall(){
    echo "I am the migrated postrm routine of $(PrometheusModule.property name)"

    systemctl stop prometheus.service prometheus_bridge.service
    systemctl disable prometheus.service prometheus_bridge.service
    rm -rf /lib/systemd/system/prometheus.service
    rm -rf /lib/systemd/system/prometheus_bridge.service

    # insert custom code
    system.unlock $(PrometheusModule.property name) "installed" $(PrometheusModule.property disableFileLock)
}

PrometheusModule.finish(){
    if [ $(system.moduleSrcDirExists $(PrometheusModule.property name)) == 1 ]; then
        echo "Detected PrometheusModule: try to restart Prometheus and Prometheus Bridge"
        systemctl enable prometheus.service prometheus_bridge.service
        systemctl restart prometheus.service prometheus_bridge.service
    fi
}
