angular.module('openITCOCKPIT')
    .controller('PrometheusQueryEditServiceController', function($scope, $stateParams, $http, $state, NotyService){

        $scope.init = true;

        $scope.serviceId = $stateParams.serviceId;
        $scope.metricsSelected = [];
        $scope.metricDetails = [];

        $scope.currentServiceStringValue = null;

        $scope.services = [];

        $scope.load = function(){
            $http.get("/prometheus_module/PrometheusQuery/editService/" + $scope.serviceId + ".json", {
                params: {
                    'angular': true
                }
            }).then(function(result){
                $scope.postData = result.data.postData;
                $scope.host = result.data.host;

                $scope.servicetemplates = result.data.servicetemplates;
                $scope.metricsSelected = result.data.selectedMetrics;

                $scope.loadMetrics('');
            });
        };

        $scope.loadMetrics = function(searchString){
            $http.get("/prometheus_module/PrometheusQuery/index.json", {
                params: {
                    'angular': true,
                    'hostId': $scope.host.id,
                    'filter[PrometheusQuery.name]': searchString,
                    'selected[]': $scope.metricsSelected
                }
            }).then(function(result){
                $scope.host = result.data.host;

                var metrics = [];

                if(result.data.targets.hasOwnProperty('status')){
                    if(result.data.targets.status === 'success'){
                        for(var index in result.data.targets.data){
                            metrics.push({
                                key: result.data.targets.data[index].metric,
                                value: result.data.targets.data[index].metric
                            });
                        }
                    }
                }

                $scope.metrics = metrics;

                if($scope.init === true){
                    $scope.loadCurrentMetricValues();
                }

                $scope.init = false;
            });
        };

        $scope.loadCurrentMetricValues = function(){
            for(var key in $scope.metricsSelected){
                $http.get("/prometheus_module/PrometheusQuery/loadCurrentValueByMetric.json", {
                    params: {
                        'hostUuid': $scope.host.uuid,
                        'metric': $scope.metricsSelected[key]
                    }
                }).then(function(result){
                    for(var index in result.data.metricDetails.data.result){
                        $scope.metricDetails.push({
                            key: result.data.metricDetails.data.result[index].metric.__name__,
                            details: result.data.metricDetails.data.result[index].metric,
                            value: result.data.metricDetails.data.result[index].value[1]
                        });
                    }

                });
            }
        };

        $scope.loadCurrentMetricValue = function(){
            $scope.queryError = null;
            if($scope.postData.Service.prometheus_alert_rule.promql.length > 0){
                $http.get("/prometheus_module/PrometheusQuery/loadValueByMetric.json", {
                    params: {
                        'metric': $scope.postData.Service.prometheus_alert_rule.promql.replace(/\s/g, '')
                    }
                }).then(function(result){
                    $scope.currentServiceStringValue = null;
                    if(result.data.metricValue.data.result.length > 0){
                        $scope.currentServiceStringValue = result.data.metricValue.data.result[0].value[1];
                    }
                }, function errorCallback(result){

                    if(result.data.hasOwnProperty('error')){
                        $scope.queryError = result.data.error;
                    }
                });
            }
        };

        $scope.addMetricString = function(parameterToAdd){
            $scope.postData.Service.prometheus_alert_rule.promql += sprintf('%1$s{uuid="%2$s"}', parameterToAdd, $scope.host.uuid);
        };

        $scope.addToString = function(parameterToAdd){
            $scope.postData.Service.prometheus_alert_rule.promql += parameterToAdd;
        };

        $scope.addAggregationToString = function(operator){
            if($scope.postData.Service.prometheus_alert_rule.promql.length === 0){
                return;
            }
            $scope.postData.Service.prometheus_alert_rule.promql = sprintf('%1$s(%2$s)', operator, trim($scope.postData.Service.prometheus_alert_rule.promql).replace(
                /\s/g, ',')
            );
        };

        $scope.editService = function(){
            //Validate data first
            $http.post("/prometheus_module/PrometheusQuery/validateService.json?angular=true",
                {
                    Service: {
                        threshold_type: $scope.postData.Service.prometheus_alert_rule.threshold_type,
                        warning_operator: $scope.postData.Service.prometheus_alert_rule.warning_operator,
                        warning_min: $scope.postData.Service.prometheus_alert_rule.warning_min,
                        warning_max: $scope.postData.Service.prometheus_alert_rule.warning_max,
                        warning_longer_as: $scope.postData.Service.prometheus_alert_rule.warning_longer_as,
                        critical_operator: $scope.postData.Service.prometheus_alert_rule.critical_operator,
                        critical_min: $scope.postData.Service.prometheus_alert_rule.critical_min,
                        critical_max: $scope.postData.Service.prometheus_alert_rule.critical_max,
                        critical_longer_as: $scope.postData.Service.prometheus_alert_rule.critical_longer_as,


                        servicetemplate_id: $scope.postData.Service.servicetemplate_id,
                        name: $scope.postData.Service.name,
                        unit: $scope.postData.Service.prometheus_alert_rule.unit,
                        promql: $scope.postData.Service.prometheus_alert_rule.promql
                    }
                }
            ).then(function(result){

                //Data is valid
                saveService();

            }, function errorCallback(result){
                //Validation error
                NotyService.genericError();

                if(result.data.hasOwnProperty('error')){
                    $scope.errors = result.data.error;

                }
            });
        };

        var saveService = function(){
            $http.post("/prometheus_module/PrometheusQuery/editService/" + $scope.serviceId + ".json?angular=true",
                $scope.postData
            ).then(function(result){
                //Data is valid
                NotyService.genericSuccess();

                //Last service. Redirect the user
                $state.go('PrometheusAlertRulesIndex', {
                    hostId: $scope.postData.Service.host_id
                });


            }, function errorCallback(result){

                //Validation error
                NotyService.genericError();

                if(result.data.hasOwnProperty('error')){
                    $scope.errors = result.data.error;
                }
            });
        };

        $scope.removeService = function(index){
            if($scope.services.length >= index){
                //Array index exists - remove service
                $scope.services.splice(index, 1);
            }
        };

        // Fire on page load
        $scope.$watch('metricsSelected', function(){
            if($scope.init){
                return;
            }

            $scope.metricDetails = [];
            $scope.currentServiceStringValue = null;
            $scope.loadCurrentMetricValues();
        }, true);

        $scope.load('');
    });
