angular.module('openITCOCKPIT')
    .controller('PrometheusQueryToServiceController', function($scope, $stateParams, $http, $state, NotyService){

        $scope.init = true;
        $scope.isCreatingServices = false;

        $scope.metricsSelected = $stateParams.metrics;
        $scope.currentServiceString = '';
        $scope.metricDetails = [];

        $scope.currentServiceStringValue = null;

        $scope.services = [];

        $scope.defaultServicetemplateId = 0;
        var clearCurrentService = function(){
            $scope.currentService = {
                threshold_type: 'scalar',
                warning_operator: null,
                warning_min: null,
                warning_max: null,
                warning_longer_as: '1m',
                critical_operator: null,
                critical_min: null,
                critical_max: null,
                critical_longer_as: '2m',


                servicetemplate_id: $scope.defaultServicetemplateId,
                name: '',
                longer_as: '1m',
                unit: '',
                promql: ''
            };
        };
        clearCurrentService();

        $scope.load = function(searchString){
            if($stateParams.hostId === null){
                // Comment this out for development!
                NotyService.info({
                    message: 'No host selected.'
                });
                $state.go('PrometheusQueryIndex');
                return;
            }

            $http.get("/prometheus_module/PrometheusQuery/index.json", {
                params: {
                    'angular': true,
                    'hostId': $stateParams.hostId,
                    'filter[PrometheusQuery.name]': searchString,
                    'selected[]': $scope.metricsSelected
                }
            }).then(function(result){
                $scope.host = result.data.host;

                var metrics = [];

                if(result.data.targets.hasOwnProperty('status')){
                    if(result.data.targets.status === 'success'){
                        for(var index in result.data.targets.data){
                            metrics.push({
                                key: result.data.targets.data[index].metric,
                                value: result.data.targets.data[index].metric
                            });
                        }
                    }
                }

                $scope.metrics = metrics;

                if($scope.init === true){
                    $scope.loadCurrentMetricValues();
                }

                $scope.init = false;
            });


            $http.get("/prometheus_module/PrometheusQuery/loadServicetemplates/" + $stateParams.hostId + ".json", {
                params: {
                    'angular': true
                }
            }).then(function(result){
                $scope.servicetemplates = result.data.servicetemplates;

                $scope.defaultServicetemplateId = result.data.servicetemplates[0].key;
                $scope.currentService.servicetemplate_id = $scope.defaultServicetemplateId;
            });
        };

        $scope.loadCurrentMetricValues = function(){
            for(var key in $scope.metricsSelected){
                $http.get("/prometheus_module/PrometheusQuery/loadCurrentValueByMetric.json", {
                    params: {
                        'hostUuid': $scope.host.uuid,
                        'metric': $scope.metricsSelected[key]
                    }
                }).then(function(result){
                    for(var index in result.data.metricDetails.data.result){
                        $scope.metricDetails.push({
                            key: result.data.metricDetails.data.result[index].metric.__name__,
                            details: result.data.metricDetails.data.result[index].metric,
                            value: result.data.metricDetails.data.result[index].value[1]
                        });
                    }

                });
            }
        };

        $scope.loadCurrentMetricValue = function(){
            $scope.queryError = null;
            if($scope.currentServiceString.length > 0){
                $http.get("/prometheus_module/PrometheusQuery/loadValueByMetric.json", {
                    params: {
                        'metric': $scope.currentServiceString.replace(/\s/g, '')
                    }
                }).then(function(result){
                    $scope.currentServiceStringValue = null;
                    if(result.data.metricValue.data.result.length > 0){
                        $scope.currentServiceStringValue = result.data.metricValue.data.result[0].value[1];
                    }
                }, function errorCallback(result){

                    if(result.data.hasOwnProperty('error')){
                        $scope.queryError = result.data.error;
                    }
                });
            }
        };

        $scope.addMetricString = function(parameterToAdd){
            $scope.currentServiceString += sprintf('%1$s{uuid="%2$s"}', parameterToAdd, $scope.host.uuid);
        };

        $scope.addToString = function(parameterToAdd){
            $scope.currentServiceString += parameterToAdd;
        };

        $scope.addAggregationToString = function(operator){
            if($scope.currentServiceString.length === 0){
                return;
            }
            $scope.currentServiceString = sprintf('%1$s(%2$s)', operator, trim($scope.currentServiceString).replace(
                /\s/g, ',')
            );
        };

        $scope.addToServices = function(){
            $scope.currentService.promql = $scope.currentServiceString;

            $http.post("/prometheus_module/PrometheusQuery/validateService.json?angular=true",
                {
                    'Service': $scope.currentService
                }
            ).then(function(result){

                //Data is valid
                $scope.services.push($scope.currentService);
                clearCurrentService();

            }, function errorCallback(result){
                //Validation error
                NotyService.genericError();

                if(result.data.hasOwnProperty('error')){
                    $scope.errors = result.data.error;

                }
            });
        };

        $scope.createServices = function(){
            $scope.isCreatingServices = true;

            for(var index in $scope.services){
                var service = $scope.services[index];
                var post = {
                    Service: {
                        host_id: $stateParams.hostId,
                        servicetemplate_id: service.servicetemplate_id,
                        name: service.name,

                        prometheus_alert_rule: {
                            host_id: $stateParams.hostId,
                            servicetemplate_id: $scope.defaultServicetemplateId,
                            promql: service.promql,
                            unit: service.unit,
                            threshold_type: service.threshold_type,
                            warning_operator: service.warning_operator,
                            warning_min: service.warning_min,
                            warning_max: service.warning_max,
                            warning_longer_as: service.warning_longer_as,
                            critical_operator: service.critical_operator,
                            critical_min: service.critical_min,
                            critical_max: service.critical_max,
                            critical_longer_as: service.critical_longer_as
                        }
                    }
                };

                $http.post("/prometheus_module/PrometheusQuery/createService.json?angular=true",
                    post
                ).then(function(result){
                    $scope.isCreatingServices = false;
                    //Data is valid
                    if((parseInt(index, 10) + 1) === $scope.services.length){
                        NotyService.genericSuccess();

                        //Last service. Redirect the user
                        $state.go('PrometheusAlertRulesIndex', {
                            hostId: $stateParams.hostId
                        });
                    }

                }, function errorCallback(result){
                    $scope.isCreatingServices = false;

                    //Validation error
                    NotyService.genericError();

                    if(result.data.hasOwnProperty('error')){
                        $scope.errors = result.data.error;
                    }
                });

            }
        };

        $scope.removeService = function(index){
            if($scope.services.length >= index){
                //Array index exists - remove service
                $scope.services.splice(index, 1);
            }
        };

        // Fire on page load
        $scope.$watch('metricsSelected', function(){
            if($scope.init){
                return;
            }

            $scope.metricDetails = [];
            $scope.currentServiceStringValue = null;
            $scope.loadCurrentMetricValues();
        }, true);

        $scope.load('');
    });
